@@ -64,8 +64,10 @@ class fgcddata :  public gcddata {
 		virtual ~fgcddata(void) { };
 		virtual const real indice(void)	const; 
 		virtual void  getpdata(partialdata *);  
-		virtual real updatecrt(direction dir,mindices& mmind,vind var,partialdata* pdt,bool& reliable,const double tol,const double) const;
-		virtual void pivot(direction dir,mindices& mmind,vind vp,vind t,partialdata* pdt,subsetdata* fdt,bool last,bool& reliable,const double tol);
+		virtual real updatecrt(direction dir,mindices& mmind,vind var,partialdata* pdt,
+			bool& reliable,const double tol,const double) const;
+		virtual void pivot(direction dir,mindices& mmind,vind vp,vind t,partialdata* pdt,subsetdata* fdt,
+					bool last,bool& reliable,const double tol);
 /* 
 	Note: partialdata and subsetdata pointer must point to partialfgcddata and fgcddata classes
 		  or unpredictable behaviour will result  
