@@ -3,7 +3,7 @@
        SUBROUTINE genetic(criterio,p,svector,kmin,kmax,valores,vars,
      +   bestval,bestvar,nfora,fora,ndentro,dentro,npopinic,nger,nclone,
      +   melhorar,mutprob,improvement,nqsi,qsi,esp,kabort,pilog,popinit,
-     +   valp, vecvecp,hvector,rh)
+     +   valp, vecvecp,hvector,rh,checksg,numtol,numprb)
 *#######################################################################
 
 * Routine to perform a Genetic Algorithm search for a k-variable subset
@@ -72,7 +72,9 @@
 *            matrix (H) of the p variables (given as a vector for
 *            convenience in passing from R to Fortran). 
 *   rh     - integer variable giving the expected rank of H matrix.
-* 
+* checksg - logical, flag indicating if a test for ill-conditioned 
+*	    (E being numerically singular) problems should be implemented
+*  numtol - double precision tolerance level for the ratio of the diagonal elements of E Choleski decomposition
 *
 * OUTPUT: 
 *
@@ -102,8 +104,10 @@
 * kabort  - integer variable indicating the cardinality in which an execution 
 *           of the algorithm may have been aborted for lack of genetic 
 *           variability (ouput for R/S).
+* numprb -  logical flag set to true if numerical problems precluded some
+*           (ill-conditioned) subsets to be evaluated, and to false otherwise.   
+*
 *
-
 
 * general declarations
        INTEGER p,kmin,kmax,criterio,npopinic,nger,nclone,poriginal
@@ -113,19 +117,19 @@
        INTEGER garanhao(npopinic),femea(npopinic),randint
        INTEGER vars(npopinic*(kmax-kmin+1)*kmax)
        INTEGER bestvar((kmax-kmin+1)*kmax),kabort
+       INTEGER popinit(kmax*npopinic*(kmax-kmin+1))
        LOGICAL setk(p),improvement 
        LOGICAL pop(npopinic,p),novager(npopinic,p),rep(p+1)
        LOGICAL filho(npopinic,p),igual,melhorar
+       LOGICAL pilog,checksg,numprb
        DOUBLE PRECISION s(p,p),sq(p,p),svector(p*p), vecvecp(p*p),h(p,p)
-       DOUBLE PRECISION vactual,vcorrente,hvector(p*p)
-       DOUBLE PRECISION valores((kmax-kmin+1)*npopinic)
+       DOUBLE PRECISION vactual,vcorrente,numtol
+       DOUBLE PRECISION valores((kmax-kmin+1)*npopinic),hvector(p*p)
        DOUBLE PRECISION valuepop(npopinic+1),valueger(0:npopinic)
        DOUBLE PRECISION critvalue,amax,aux,somavalue,acumulado,vantes
        DOUBLE PRECISION vmaximo,valuenovager(npopinic)
        DOUBLE PRECISION bestval(kmax-kmin+1)
        DOUBLE PRECISION testran, mutprob,r,ranaux,unifrnd
-       LOGICAL pilog
-       INTEGER popinit(kmax*npopinic*(kmax-kmin+1))
 * declarations only for the RM criterion
        DOUBLE PRECISION tracos,dobjrm
 * declarations only for the RV criterion
@@ -134,20 +138,23 @@
        DOUBLE PRECISION valp(p), vecp(p,p),dobjgcd
        INTEGER nqsi,qsi(p)
        LOGICAL esp
-
 * declarations for tau2,xi2,zeta2 e ccr12
        Double precision dobjtau2,dobjxi2,dobjzeta2,dobjccr12
+* declarations of local arrays and matrices
+       INTEGER randsk1pp(p),setint(p)
+       DOUBLE PRECISION skinput(kmax,kmax),hkinput(kmax,kmax)
+       DOUBLE PRECISION ekinput(kmax,kmax),workmat(kmax,kmax)
+       DOUBLE PRECISION work(6*kmax),egval(kmax)
 
-
-       external randsk1,dobjrm,dobjrv,dobjgcd
-       external dcorrigesk
-       external dmelhoramentogen,newinicializar
-       external intpr,dblepr
-       external rndstart,rndend,unifrnd
+       external dobjrm,dobjrv,dobjgcd
        external dobjtau2,dobjxi2,dobjzeta2,dobjccr12
+       external dcorrigesk,dmelhoramentogen,newinicializar
+       external intpr,dblepr
+       external randsk1,rndstart,rndend,unifrnd
 
 * initializations
-       critvalue = 0.0D0
+       critvalue = -1.0D0
+       numprb=.false.
 
        call rndstart()       
        call newinicializar(criterio,p,s,svector,sq,nfora,fora,ndentro,
@@ -157,7 +164,6 @@
       kabort = kmax+1
 
 
-
 **********************************
 * The loop which is to be repeated for each cardinality of subsets, 
 * between kmin e kmax starts here and ends together with the subroutine.
@@ -181,7 +187,6 @@
 * represented in the initial population, in which case rep(i)=.true. 
 * for i=1,...,p
 
-
          do i=1,p
             rep(i)=.false.
          end do
@@ -204,7 +209,7 @@
                end do
              end if  
            else
-               call randsk1(p-ndentro,k-ndentro,setk)           
+               call randsk1(p-ndentro,k-ndentro,setk,randsk1pp)           
                if(ndentro .GT. 0) then
                 call dcorrigesk(ndentro,dentro,p,setk,poriginal)
                endif
@@ -246,41 +251,46 @@
            end do
 
            if (criterio.eq.1) then
-               amax = dobjrm(k,setk,p,s,sq,poriginal)
+		amax = dobjrm(k,setk,p,poriginal,kmax,s,sq,workmat,
+     +   setint,skinput)
                critvalue = dsqrt(amax/tracos)
            end if
-
            if (criterio.eq.2) then
-               amax = dobjrv(k,setk,p,s,sq,poriginal)
+               amax = dobjrv(k,setk,p,poriginal,kmax,s,sq,workmat,
+     +   setint,skinput)
                critvalue = dsqrt(amax/tracosq)
            end if
            if (criterio.eq.3) then
-            amax = dobjgcd(nqsi,qsi,valp,vecp,k,setk,p,s,fica,poriginal)
+	     amax = dobjgcd(k,setk,p,poriginal,kmax,s,workmat,
+     +   nqsi,qsi,valp,setint,vecp,fica,skinput)
             critvalue = amax/dsqrt(DBLE(nqsi*k))
            end if
-
            if (criterio.eq.4) then
-               amax=dobjtau2(k,setk,p,s,poriginal,h,rh)
+               amax=dobjtau2(k,setk,p,poriginal,kmax,s,h,rh,checksg,
+     +   numtol,setint,work,egval,skinput,hkinput,ekinput,workmat)
                critvalue = amax
-           end if
-		 
+           end if		 
            if (criterio.eq.5) then
-               amax=dobjxi2(k,setk,p,s,poriginal,h,rh)
+               amax=dobjxi2(k,setk,p,poriginal,kmax,s,h,rh,checksg,
+     +   numtol,setint,work,egval,skinput,hkinput,workmat)
                critvalue = amax
-           end if
-		 
+           end if		 
 	   if (criterio.eq.6) then
-               amax=dobjzeta2(k,setk,p,s,poriginal,h,rh)
+               amax=dobjzeta2(k,setk,p,poriginal,kmax,s,h,rh,checksg,
+     +   numtol,setint,work,egval,skinput,hkinput,ekinput,workmat)
 	       critvalue = amax
-           end if
-		 
+           end if		 
 	   if (criterio.eq.7) then
 * tirei rh
-               amax=dobjccr12(k,setk,p,s,poriginal,h)
+               amax=dobjccr12(k,setk,p,poriginal,kmax,s,h,checksg,
+     +   numtol,setint,work,egval,skinput,hkinput,workmat)
 	       critvalue = amax
            end if
-		 
 
+           if (checksg.and.criterio.GT.3.and.vtroca.eq.-0.9999D0) then
+	       numprb=.true.        
+           end if
+	 
 	   valueger(kpop) = critvalue
            somavalue=somavalue+valueger(kpop)
            aux=valueger(kpop)
@@ -365,9 +375,34 @@
                 call intpr(' Try reducing the maximum acceptable number
      + of clones (maxclone) or increasing the population size (popsize)'
      + , -1, k,0)
-                call dblepr(' Best criterion value found so far:', -1,
-     + valuepop(1),1)
+                if (valuepop(1).GE.0.0D0) then
+		 call dblepr(' Best criterion value found so far:',
+     + -1,valuepop(1),1)
+		end if 
                 kabort = k
+
+*  preparing the output which will be used in R.
+                do kpop=1,npop
+                  jjaux=0
+                  do i=1,p
+                    if (pop(kpop,i)) then
+                      jjaux=jjaux+1
+		      if (valuepop(kpop).LT.0.0D0) then 
+                        vars(npop*kmax*(k-kmin)+npop*(jjaux-1)+kpop)=0
+		      else
+                        vars(npop*kmax*(k-kmin)+npop*(jjaux-1)+kpop) = 
+     +                 fica(i)
+		      end if
+                    end if
+                  end do  
+                  valores((k-kmin)*npop+kpop) = valuepop(kpop) 
+                end do
+                bestval(k-kmin+1)=valuepop(1) 
+         	do j=1,k
+                   bestvar(kmax*(k-kmin)+j) =
+     +           vars(npop*kmax*(k-kmin)+npop*(j-1)+1)
+                end do
+
 *                call rndend()
                 return
 	       end if   
@@ -410,12 +445,12 @@
 	       end if
               end do
               kpainaomae=randint(1,npainaomae-1)
-              call randsk1(npainaomae,kpainaomae,setk)
+              call randsk1(npainaomae,kpainaomae,setk,randsk1pp)
               do i=1,npainaomae
                 filho(kfilho,painaomae(i))=setk(i)
               end do
               kmaenaopai=npainaomae-kpainaomae
-              call randsk1(nmaenaopai,kmaenaopai,setk)
+             call randsk1(nmaenaopai,kmaenaopai,setk,randsk1pp)
               do i=1,nmaenaopai
                 filho(kfilho,maenaopai(i))=setk(i)
               end do
@@ -456,9 +491,9 @@
 	        kfilhoant=kfilhoant+1
 	      end do
               if(nvezes.GT.nclone) then
-              call randsk1(p-ndentro,k-ndentro,setk)
+              call randsk1(p-ndentro,k-ndentro,setk,randsk1pp)
               if(ndentro.GT.0) then 
-                  call dcorrigesk(ndentro,dentro,p,setk,poriginal)
+                 call dcorrigesk(ndentro,dentro,p,setk,poriginal)
               endif
               do j=1,p
                 filho(kfilho,j)=setk(j)
@@ -476,33 +511,39 @@
 	        setk(j)=filho(kfilho,j)
               end do
               if (criterio.eq.1) then
-                 amax = dobjrm(k,setk,p,s,sq,poriginal)
+		  amax = dobjrm(k,setk,p,poriginal,kmax,s,sq,workmat,
+     +   setint,skinput)
                  critvalue = dsqrt(amax/tracos)
               end if
               if (criterio.eq.2) then
-                 amax = dobjrv(k,setk,p,s,sq,poriginal)
+                  amax = dobjrv(k,setk,p,poriginal,kmax,s,sq,workmat,
+     +   setint,skinput)
                  critvalue = dsqrt(amax/tracosq)
               end if
               if (criterio.eq.3) then
-                 amax = dobjgcd(nqsi,qsi,valp,vecp,k,setk,p,s,
-     +                   fica,poriginal)
+		 amax = dobjgcd(k,setk,p,poriginal,kmax,s,workmat,
+     +   nqsi,qsi,valp,setint,vecp,fica,skinput)
                  critvalue = amax/dsqrt(DBLE(nqsi*k))
               end if
               if (criterio.eq.4) then
-                 amax=dobjtau2(k,setk,p,s,poriginal,h,rh)
+                 amax=dobjtau2(k,setk,p,poriginal,kmax,s,h,rh,checksg,
+     +   numtol,setint,work,egval,skinput,hkinput,ekinput,workmat)
                  critvalue = amax
               end if 
 	      if (criterio.eq.5) then
-                 amax=dobjxi2(k,setk,p,s,poriginal,h,rh)
+                 amax=dobjxi2(k,setk,p,poriginal,kmax,s,h,rh,checksg,
+     +   numtol,setint,work,egval,skinput,hkinput,workmat)
                  critvalue = amax
               end if
 	      if (criterio.eq.6) then
-                 amax=dobjzeta2(k,setk,p,s,poriginal,h,rh)
-	         critvalue = amax
+                amax=dobjzeta2(k,setk,p,poriginal,kmax,s,h,rh,checksg,
+     +   numtol,setint,work,egval,skinput,hkinput,ekinput,workmat)
+	        critvalue = amax
               end if
               if (criterio.eq.7) then
 * tirei rh
-                 amax=dobjccr12(k,setk,p,s,poriginal,h)
+                 amax=dobjccr12(k,setk,p,poriginal,kmax,s,h,checksg,
+     +   numtol,setint,work,egval,skinput,hkinput,workmat)
 	         critvalue = amax
               end if
 	      valueger(kfilho)=critvalue
@@ -514,14 +555,15 @@
 * and slow down computation times considerably if 
 * "mutprob" is large. Default value for "melhorar" is .false.
 
-              if(melhorar) then
-*               call rndstart()
+              if(melhorar.and.critvalue.GE.0.0D0) then
                testran = unifrnd()
-*               call rndend()
                if (testran .LT. mutprob) then
                 vantes=amax
-                call dmelhoramentogen(criterio,p,setk,amax,ndentro,
-     +          dentro,k,s,sq,nqsi,qsi,valp,vecp,fica,poriginal,h,rh)
+                call dmelhoramentogen(criterio,p,kmax,setk,vactual,
+     +                ndentro,dentro,k,s,sq,nqsi,qsi,valp,vecp,fica,
+     +                poriginal,h,rh,skinput,hkinput,ekinput,egval,
+     +                work,workmat,randsk1pp,setint,
+     +                checksg,numtol,numprb)
                 if(amax.GT.vantes) then
                   if (criterio.eq.1) then
                     critvalue = dsqrt(amax/tracos)
@@ -630,14 +672,20 @@
            do i=1,p
              if (pop(kpop,i)) then
                jjaux=jjaux+1
-               vars(npop*kmax*(k-kmin)+npop*(jjaux-1)+kpop) = fica(i)
+		if (critvalue.LT.0.0D0) then 
+                   vars(npop*kmax*(k-kmin)+npop*(jjaux-1)+kpop) = 0
+		else
+                   vars(npop*kmax*(k-kmin)+npop*(jjaux-1)+kpop) = 
+     +           fica(i)
+		end if
              end if
            end do  
            valores((k-kmin)*npop+kpop) = valuepop(kpop) 
           end do
 
 * Optional further improvement of the best solution
-         if (improvement) then
+*         if (improvement) then
+         if (improvement.and.valuepop(1).GE.0.0D0) then
            if (criterio.eq.1) vactual=valuepop(1)**2*tracos
            if (criterio.eq.2) vactual=valuepop(1)**2*tracosq
            if (criterio.eq.3) vactual=valuepop(1)*dsqrt(DBLE(nqsi*k))
@@ -650,9 +698,10 @@
            end do
            vcorrente = vactual
 
-           call dmelhoramentogen(criterio,p,setk,vactual,ndentro,dentro,
-     +      k,s,sq,nqsi,qsi,valp,vecp,fica,poriginal,h,rh)
-
+	   call dmelhoramentogen(criterio,p,kmax,setk,vactual,
+     +         ndentro,dentro,k,s,sq,nqsi,qsi,valp,vecp,fica,
+     +         poriginal,h,rh,skinput,hkinput,ekinput,egval,
+     +         work,workmat,randsk1pp,setint,checksg,numtol,numprb)
 
            if (vactual .GT. vcorrente) then 
                    naux=0
@@ -694,9 +743,9 @@
          end do
          end do 
 
+
 * End of k-loop
 
-*         call rndend()
          call rndend()
          return
        end
